from app import db, logger
from flask import current_app, flash, redirect, render_template, url_for
from flask_login import current_user, login_required, logout_user
from threading import Thread
from . import profile
from .background_functions import delete_user_
from .forms import EditEmailForm, EditGeneralSettingsForm, EditPasswordForm


@profile.route('/settings', methods=['GET', 'POST'])
@login_required
def settings():
    edit_email_form = EditEmailForm(prefix='edit-email-form')
    edit_general_settings_form = EditGeneralSettingsForm(
        prefix='edit-general-settings-form'
    )
    edit_password_form = EditPasswordForm(prefix='edit-password-form',
                                          user=current_user)
    # Check if edit_email_form is submitted and valid
    if (edit_email_form.save_email.data
            and edit_email_form.validate_on_submit()):
        logger.warning('edit_email_form submitted')
        db.session.add(current_user)
        db.session.commit()
        flash('Your email address has been updated.')
        return redirect(url_for('profile.settings'))
    # Check if edit_settings_form is submitted and valid
    if (edit_general_settings_form.save_settings.data
            and edit_general_settings_form.validate_on_submit()):
        logger.warning('edit_general_settings_form submitted')
        current_user.is_dark = edit_general_settings_form.dark_mode.data
        db.session.add(current_user)
        db.session.commit()
        flash('Your settings have been updated.')
        return redirect(url_for('profile.settings'))
    # Check if edit_password_form is submitted and valid
    if (edit_password_form.save_password.data
            and edit_password_form.validate_on_submit()):
        logger.warning('edit_password_form submitted')
        current_user.password = edit_password_form.password.data
        db.session.add(current_user)
        db.session.commit()
        flash('Your password has been updated.')
        return redirect(url_for('profile.settings'))
    # If no form is submitted or valid, fill out fields with current values
    logger.warning('nothing submitted')
    edit_email_form.email.data = current_user.email
    edit_general_settings_form.dark_mode.data = current_user.is_dark
    return render_template(
        'profile/settings.html.j2',
        edit_email_form=edit_email_form,
        edit_password_form=edit_password_form,
        edit_general_settings_form=edit_general_settings_form,
        title='Settings'
    )


@profile.route('/delete', methods=['GET', 'POST'])
@login_required
def delete():
    """
    View to delete yourslef and all associated data.
    """
    thread = Thread(target=delete_user_,
                    args=(current_app._get_current_object(), current_user.id))
    thread.start()
    logout_user()
    flash('Your account has been deleted!')
    return redirect(url_for('main.index'))
